from cs_dynamicpages import PACKAGE_NAME

import pytest


class TestSetupUninstall:
    @pytest.fixture(autouse=True)
    def uninstalled(self, installer):
        installer.uninstall_product(PACKAGE_NAME)

    def test_addon_uninstalled(self, installer):
        """Test if cs_dynamicpages is uninstalled."""
        assert installer.is_product_installed(PACKAGE_NAME) is False

    def test_browserlayer_not_registered(self, browser_layers):
        """Test that IBrowserLayer is not registered."""
        from cs_dynamicpages.interfaces import IBrowserLayer

        assert IBrowserLayer not in browser_layers



from cs_dynamicpages import _
from plone import schema
from plone.app.z3cform.widgets.link import LinkFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import safe_hasattr
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class ILinkInfoMarker(Interface):
    pass


@provider(IFormFieldProvider)
class ILinkInfo(model.Schema):
    """ """

    link_text = schema.TextLine(
        title=_("Link text"),
        description=_("Enter the text that will be linked"),
        required=False,
    )

    link_url = schema.TextLine(
        title=_("Link URL"),
        description=_("Enter the URL of the link"),
        required=False,
    )

    form.widget(
        "link_url",
        LinkFieldWidget,
    )


@implementer(ILinkInfo)
@adapter(ILinkInfoMarker)
class LinkInfo:
    def __init__(self, context):
        self.context = context

    @property
    def link_text(self):
        if safe_hasattr(self.context, "link_text"):
            return self.context.link_text
        return None

    @link_text.setter
    def link_text(self, value):
        self.context.link_text = value

    @property
    def link_url(self):
        if safe_hasattr(self.context, "link_url"):
            return self.context.link_url
        return None

    @link_url.setter
    def link_url(self, value):
        self.context.link_url = value



from plone import api
from plone.app.layout.globals.layout import IBodyClassAdapter
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@adapter(Interface, Interface)
@implementer(IBodyClassAdapter)
class DynamicViewFolderClasses:
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_classes(self, template, view):
        """Whenever we are in a dynamic-view, add a custom class
        signaling that the user can edit the content
        """
        if template.id == "dynamic_view.pt":
            can_edit = api.user.has_permission(
                "Modify portal content", obj=self.context
            )
            if can_edit:
                return ["can_edit"]
            return []
        return []



from base64 import b64decode
from collections import defaultdict
from cs_dynamicpages.interfaces import IBrowserLayer
from plone import api
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.namedfile import NamedBlobImage
from plone.testing.zope import Browser
from zope.interface import alsoProvides

import pytest


@pytest.fixture
def contents() -> list:
    """Content to be created."""
    return [
        # EU CONTENT
        {
            "_container": "",
            "_language": "eu",
            "_transitions": ["publish"],
            "type": "Folder",
            "id": "folder",
            "title": "Folder",
        },
        {
            "_container": "folder",
            "_transitions": ["publish"],
            "_language": "eu",
            "type": "DynamicPageFolder",
            "id": "dpf",
            "title": "DPF",
        },
        {
            "_container": "folder/dpf",
            "_transitions": ["publish"],
            "_language": "eu",
            "type": "DynamicPageRow",
            "id": "row-1",
            "title": "Row 1",
        },
        {
            "_container": "folder/dpf",
            "_transitions": ["publish"],
            "_language": "eu",
            "type": "DynamicPageRow",
            "id": "row-2",
            "title": "Row 2",
        },
    ]


@pytest.fixture()
def portal(functional):
    return functional["portal"]


@pytest.fixture()
def browser(functional):
    browser = Browser(functional["app"])
    browser.handleErrors = False
    browser.addHeader("Authorization", f"Basic {SITE_OWNER_NAME}:{SITE_OWNER_PASSWORD}")
    return browser


@pytest.fixture()
def my_request(functional):
    req = functional["request"]
    alsoProvides(req, IBrowserLayer)
    return req


@pytest.fixture
def create_contents(contents):
    """Helper fixture to create initial content."""

    def func(portal) -> dict:
        ids = defaultdict(list)
        for item in contents:
            container_path = item["_container"]
            container = portal.unrestrictedTraverse(container_path)
            payload = {"container": container, "language": item["_language"]}
            if "_image" in item:
                payload["image"] = NamedBlobImage(b64decode(item["_image"]))
            for key, value in item.items():
                if key.startswith("_"):
                    continue
                payload[key] = value

            content = api.content.create(**payload)
            content.language = payload["language"]
            if "_view" in item:
                content.setLayout(item["_view"])
            # Set translation
            if "_translation_of" in item:
                source = portal.unrestrictedTraverse(item["_translation_of"])
                ITranslationManager(source).register_translation(
                    content.language, content
                )
            # Transition items
            if "_transitions" in item:
                transitions = item["_transitions"]
                for transition in transitions:
                    api.content.transition(content, transition=transition)
            ids[container_path].append(content.getId())
        return ids

    return func


@pytest.fixture()
def portal_with_content(app, portal, create_contents):
    """Plone portal with initial content."""
    with api.env.adopt_roles(["Manager"]):
        create_contents(portal)
    # transaction.commit()
    yield portal
    # with api.env.adopt_roles(["Manager"]):
    #     containers = sorted(content_ids.keys(), reverse=True)
    #     for container_path in containers:
    #         container = portal.unrestrictedTraverse(container_path)
    #         container.manage_delObjects(content_ids[container_path])
    # transaction.commit()



"""Init and utils."""

from zope.i18nmessageid import MessageFactory

import logging


__version__ = "1.0.0b4"

PACKAGE_NAME = "cs_dynamicpages"

_ = MessageFactory(PACKAGE_NAME)

logger = logging.getLogger(PACKAGE_NAME)



from cs_dynamicpages import _
from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import safe_hasattr
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class IRowWidthMarker(Interface):
    pass


@provider(IFormFieldProvider)
class IRowWidth(model.Schema):
    width = schema.Choice(
        title=_("Row width"),
        description=_("Select the width that this row will have"),
        vocabulary="cs_dynamicpages.RowWidth",
        required=True,
        default="col-md-12",
    )


@implementer(IRowWidth)
@adapter(IRowWidthMarker)
class RowWidth:
    def __init__(self, context):
        self.context = context

    @property
    def width(self):
        if safe_hasattr(self.context, "width"):
            return self.context.width
        return None

    @width.setter
    def width(self, value):
        self.context.width = value



from cs_dynamicpages.testing import CS_DYNAMICPAGES_FUNCTIONAL_TESTING
from cs_dynamicpages.testing import CS_DYNAMICPAGES_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class SubscriberIntegrationTest(unittest.TestCase):
    layer = CS_DYNAMICPAGES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])


class SubscriberFunctionalTest(unittest.TestCase):
    layer = CS_DYNAMICPAGES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])



# from cs_dynamicpages import _
from cs_dynamicpages.views.dynamic_page_row_view import DynamicPageRowView
from plone import api
from zope.interface import implementer
from zope.interface import Interface


class ISliderView(Interface):
    """Marker Interface for ISliderView"""


@implementer(ISliderView)
class SliderView(DynamicPageRowView):
    # If you want to define a template here, please remove the template from
    # the configure.zcml registration of this view.
    # template = ViewPageTemplateFile('slider_view.pt')

    def elements(self):
        # Implement your own actions:
        return api.content.find(
            portal_type="DynamicPageRowFeatured",
            context=self.context,
            sort_on="getObjPositionInParent",
        )



from cs_dynamicpages.testing import ACCEPTANCE_TESTING
from cs_dynamicpages.testing import FUNCTIONAL_TESTING
from cs_dynamicpages.testing import INTEGRATION_TESTING
from pytest_plone import fixtures_factory


pytest_plugins = ["pytest_plone"]


globals().update(
    fixtures_factory((
        (ACCEPTANCE_TESTING, "acceptance"),
        (FUNCTIONAL_TESTING, "functional"),
        (INTEGRATION_TESTING, "integration"),
    ))
)



from cs_dynamicpages import _
from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import safe_hasattr
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class IRowColumnsMarker(Interface):
    pass


@provider(IFormFieldProvider)
class IRowColumns(model.Schema):
    """ """

    columns = schema.Choice(
        title=_("Column count"),
        description=_("Select how many columns will be shown in this row"),
        vocabulary="cs_dynamicpages.RowColumns",
        required=True,
        default="col-md-6",
    )


@implementer(IRowColumns)
@adapter(IRowColumnsMarker)
class RowColumns:
    def __init__(self, context):
        self.context = context

    @property
    def columns(self):
        if safe_hasattr(self.context, "columns"):
            return self.context.columns
        return None

    @columns.setter
    def columns(self, value):
        self.context.columns = value



from ..base import TestBase
from cs_dynamicpages import PACKAGE_NAME


class TestSetupInstall(TestBase):
    def test_addon_installed(self, installer):
        """Test if cs_dynamicpages is installed."""
        assert installer.is_product_installed(PACKAGE_NAME) is True

    def test_browserlayer(self, browser_layers):
        """Test that IBrowserLayer is registered."""
        from cs_dynamicpages.interfaces import IBrowserLayer

        assert IBrowserLayer in browser_layers

    def test_latest_version(self, profile_last_version):
        """Test latest version of default profile."""
        assert profile_last_version(f"{PACKAGE_NAME}:default") == "1007"



from cs_dynamicpages.behaviors.row_width import IRowWidthMarker
from cs_dynamicpages.testing import CS_DYNAMICPAGES_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.behavior.interfaces import IBehavior
from zope.component import getUtility

import unittest


class RowWidthIntegrationTest(unittest.TestCase):
    layer = CS_DYNAMICPAGES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_behavior_row_width(self):
        behavior = getUtility(IBehavior, "cs_dynamicpages.row_width")
        self.assertEqual(
            behavior.marker,
            IRowWidthMarker,
        )



from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing.zope import WSGI_SERVER_FIXTURE

import cs_dynamicpages


class Layer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=cs_dynamicpages)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "cs_dynamicpages:default")


FIXTURE = Layer()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name="CsDynamicpagesLayer:IntegrationTesting",
)


FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE, WSGI_SERVER_FIXTURE),
    name="CsDynamicpagesLayer:FunctionalTesting",
)


ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        WSGI_SERVER_FIXTURE,
    ),
    name="CsDynamicpagesLayer:AcceptanceTesting",
)



# from plone.app.textfield import RichText
# from plone.autoform import directives
from plone.dexterity.content import Container

# from plone.namedfile import field as namedfile
from plone.supermodel import model

# from plone.supermodel.directives import fieldset
# from z3c.form.browser.radio import RadioFieldWidget
# from zope import schema
from zope.interface import implementer


# from cs_dynamicpages import _


class IDynamicPageFolder(model.Schema):
    """Marker interface and Dexterity Python Schema for DynamicPageFolder"""


@implementer(IDynamicPageFolder)
class DynamicPageFolder(Container):
    """Content-type class for IDynamicPageFolder"""



from cs_dynamicpages.content.dynamic_page_row import IDynamicPageRow
from cs_dynamicpages.testing import CS_DYNAMICPAGES_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryUtility

import unittest


class DynamicPageRowIntegrationTest(unittest.TestCase):
    layer = CS_DYNAMICPAGES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        portal_types = self.portal.portal_types
        parent_id = portal_types.constructContent(
            "DynamicPageFolder",
            self.portal,
            "parent_container",
            title="Parent container",
        )
        self.parent = self.portal[parent_id]

    def test_ct_dynamic_page_row_schema(self):
        fti = queryUtility(IDexterityFTI, name="DynamicPageRow")
        schema = fti.lookupSchema()
        self.assertEqual(IDynamicPageRow, schema)

    def test_ct_dynamic_page_row_fti(self):
        fti = queryUtility(IDexterityFTI, name="DynamicPageRow")
        self.assertTrue(fti)

    def test_ct_dynamic_page_row_factory(self):
        fti = queryUtility(IDexterityFTI, name="DynamicPageRow")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            IDynamicPageRow.providedBy(obj),
            f"IDynamicPageRow not provided by {obj}!",
        )

    def test_ct_dynamic_page_row_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        obj = api.content.create(
            container=self.parent,
            type="DynamicPageRow",
            id="dynamic_page_row",
        )

        self.assertTrue(
            IDynamicPageRow.providedBy(obj),
            f"IDynamicPageRow not provided by {obj.id}!",
        )

        parent = obj.__parent__
        self.assertIn("dynamic_page_row", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=obj)
        self.assertNotIn("dynamic_page_row", parent.objectIds())

    def test_ct_dynamic_page_row_globally_not_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="DynamicPageRow")
        self.assertFalse(fti.global_allow, f"{fti.id} is globally addable!")



from . import logger
from cs_dynamicpages.utils import add_custom_view
from plone import api


def upgrade(setup_tool=None):
    """ """
    logger.info("Running upgrade (Python): Remove unneeded behaviors")

    registry_values = api.portal.get_registry_record(
        "cs_dynamicpages.dynamic_pages_control_panel.row_type_fields"
    )
    new_registry_values = []
    for registry_value in registry_values:
        if not registry_value.get("row_type_icon"):
            registry_value["row_type_icon"] = "bricks"
        new_registry_values.append(registry_value)
    api.portal.set_registry_record(
        "cs_dynamicpages.dynamic_pages_control_panel.row_type_fields",
        new_registry_values,
    )

    view_names = [view["row_type"] for view in new_registry_values]
    if "cs_dynamicpages-title-description-view" not in view_names:
        add_custom_view(
            "cs_dynamicpages-title-description-view",
            [
                "IBasic.title",
                "IBasic.description",
                "IRowWidth.width",
                "IExtraClass.extra_class",
            ],
            has_button=False,
            icon="fonts",
        )
        logger.info("Added new view")

    logger.info("Upgrade step run")



# from cs_dynamicpages import _
from cs_dynamicpages import _

# from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from cs_dynamicpages.utils import get_available_views_for_row
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from uuid import uuid4
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import Interface


class IDynamicPageFolderView(Interface):
    """Marker Interface for IDynamicPageFolderView"""


@implementer(IDynamicPageFolderView)
class DynamicPageFolderView(BrowserView):
    # If you want to define a template here, please remove the template from
    # the configure.zcml registration of this view.
    # template = ViewPageTemplateFile('dynamic_page_folder_view.pt')

    def __call__(self):
        # Implement your own actions:
        return self.index()


@implementer(IDynamicPageFolderView)
class DynamicPageAddRowContentView(BrowserView):
    # If you want to define a template here, please remove the template from
    # the configure.zcml registration of this view.
    # template = ViewPageTemplateFile('dynamic_page_folder_view.pt')

    def __call__(self):
        # Implement your own actions:
        row_type = self.request.get("row_type")
        if row_type:
            random_id = uuid4()

            alsoProvides(self.request, IDisableCSRFProtection)
            api.content.create(
                type="DynamicPageRow",
                container=self.context,
                row_type=row_type,
                title=row_type,
                description="Here goes the description",
                id=str(random_id),
                link_text="Link Text",
                link_url="/",
            )
            available_views = get_available_views_for_row()
            for view in available_views:
                if view["row_type"] == row_type:
                    has_featured_button = view["row_type_has_featured_add_button"]
                    if has_featured_button:
                        created_elements_find = api.content.find(
                            portal_type="DynamicPageRow",
                            id=str(random_id),
                        )
                        created_element = created_elements_find[0].getObject()
                        random_id_featured = uuid4()
                        api.content.create(
                            type="DynamicPageRowFeatured",
                            container=created_element,
                            title="New Featured",
                            description="Here goes the description",
                            id=str(random_id_featured),
                            link_text="Link Text",
                            link_url="/",
                        )

                        random_id_featured_2 = uuid4()
                        api.content.create(
                            type="DynamicPageRowFeatured",
                            container=created_element,
                            title="New Featured 2",
                            description="Here goes the description",
                            id=str(random_id_featured_2),
                            link_text="Link Text",
                            link_url="/",
                        )
            statusmessage = _("Row added successfully")
            IStatusMessage(self.request).add(statusmessage, type="info")
            return self.request.response.redirect(
                f"{self.context.aq_parent.absolute_url()}#{random_id!s}"
            )



# from cs_dynamicpages import _
from cs_dynamicpages.views.dynamic_page_row_view import DynamicPageRowView
from plone.app.contenttypes.browser.collection import CollectionView
from zope.interface import implementer
from zope.interface import Interface


# from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class IQueryColumnsView(Interface):
    """Marker Interface for IQueryColumnsView"""


@implementer(IQueryColumnsView)
class QueryColumnsView(CollectionView, DynamicPageRowView):
    # If you want to define a template here, please remove the template from
    # the configure.zcml registration of this view.
    # template = ViewPageTemplateFile('query_columns_view.pt')

    def __call__(self):
        # Implement your own actions:
        return self.index()



# from plone.app.textfield import RichText
# from plone.autoform import directives
from cs_dynamicpages.utils import absolute_target_url
from plone import api
from plone.app.contenttypes.utils import replace_link_variables_by_paths
from plone.dexterity.content import Item

# from plone.namedfile import field as namedfile
from plone.supermodel import model

# from plone.supermodel.directives import fieldset
# from z3c.form.browser.radio import RadioFieldWidget
# from zope import schema
from zope.interface import implementer


# from cs_dynamicpages import _


class IDynamicPageRowFeatured(model.Schema):
    """Marker interface and Dexterity Python Schema for DynamicPageRowFeatured"""


@implementer(IDynamicPageRowFeatured)
class DynamicPageRowFeatured(Item):
    """Content-type class for IDynamicPageRowFeatured"""

    def review_state(self):
        return api.content.get_state(obj=self)

    def related_image_object(self):
        related_image = self.related_image
        if related_image:
            return related_image[0].to_object
        return None

    def url(self):
        """Returns the url with link variables replaced."""
        url = replace_link_variables_by_paths(self, self.link_url.strip())
        return absolute_target_url(url)



from cs_dynamicpages import _
from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import safe_hasattr
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class IFetchPriorityImageMarker(Interface):
    pass


@provider(IFormFieldProvider)
class IFetchPriorityImage(model.Schema):
    """ """

    fetchpriority_image = schema.Bool(
        title=_("Fetch priority image"),
        description=_(
            "Set this to true if you want to signal that this image has a 'high' fetch "
            "priority, otherwise 'auto' will be used. When using 'high' the value of "
            "the loading attribute will be set to 'eager', and 'lazy' when "
            "it is not set."
        ),
        required=False,
    )


@implementer(IFetchPriorityImage)
@adapter(IFetchPriorityImageMarker)
class FetchPriorityImage:
    def __init__(self, context):
        self.context = context

    @property
    def fetchpriority_image(self):
        if safe_hasattr(self.context, "fetchpriority_image"):
            return self.context.fetchpriority_image
        return None

    @fetchpriority_image.setter
    def fetchpriority_image(self, value):
        self.context.fetchpriority_image = value



from cs_dynamicpages import _
from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import safe_hasattr
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class IExtraClassMarker(Interface):
    pass


@provider(IFormFieldProvider)
class IExtraClass(model.Schema):
    """ """

    extra_class = schema.TextLine(
        title=_("Extra CSS class"),
        description=_("Enter an extra CSS class that will be added to the row"),
        required=False,
    )


@implementer(IExtraClass)
@adapter(IExtraClassMarker)
class ExtraClass:
    def __init__(self, context):
        self.context = context

    @property
    def extra_class(self):
        if safe_hasattr(self.context, "extra_class"):
            return self.context.extra_class
        return None

    @extra_class.setter
    def extra_class(self, value):
        self.context.extra_class = value



# from cs_dynamicpages import _
from cs_dynamicpages.utils import get_available_views_for_row
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from Products.Five.browser import BrowserView
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import Interface


class IDynamicView(Interface):
    """Marker Interface for IDynamicView"""


@implementer(IDynamicView)
class DynamicView(BrowserView):
    def rows(self):
        dynamic_page_folder = self.dynamic_page_folder_element()
        if dynamic_page_folder:
            return api.content.find(
                portal_type="DynamicPageRow",
                sort_on="getObjPositionInParent",
                context=dynamic_page_folder,
            )
        return []

    def dynamic_page_folder_element(self):
        page_folders = api.content.find(
            portal_type="DynamicPageFolder",
            context=self.context,
            depth=1,
            sort_on="getObjPositionInParent",
        )
        if page_folders:
            return page_folders[0].getObject()
        else:
            if self.can_edit():
                alsoProvides(self.request, IDisableCSRFProtection)
                api.content.create(
                    container=self.context,
                    type="DynamicPageFolder",
                    title="Rows",
                )
                created_elements_find = api.content.find(
                    portal_type="DynamicPageFolder",
                    context=self.context,
                    depth=1,
                    sort_on="getObjPositionInParent",
                )
                created_element = created_elements_find[0].getObject()
                api.content.transition(created_element, transition="publish")
                return created_element

    def dynamic_page_folder_element_url(self):
        dynamic_page_folder = self.dynamic_page_folder_element()
        if dynamic_page_folder:
            return dynamic_page_folder.absolute_url()
        return ""

    def can_edit(self):
        return api.user.has_permission("Modify portal content", obj=self.context)

    def available_views_for_row(self):
        return get_available_views_for_row()

    def normalize_title(self, title):
        return (
            title.replace("cs_dynamicpages-", " ")
            .replace("-", " ")
            .replace("_", " ")
            .replace("view", "")
            .lower()
        )



from plone import api
from plone.app.dexterity import textindexer
from plone.dexterity.interfaces import IDexterityContainer
from zope.component import adapter
from zope.interface import implementer


def extract_text_value_to_index(content):
    """convert the text field of a content item to plain text"""
    text = (content.text and content.text.output) or ""
    pt = api.portal.get_tool("portal_transforms")
    data = pt.convertTo("text/plain", text, mimetype="text/html")
    return data.getData()


FIELDS_TO_INDEX = {
    # Full field name: function to get the indexed value
    "IBasic.title": lambda content: content.Title(),
    "IBasic.description": lambda content: content.Description(),
    "IRichTextBehavior-text": extract_text_value_to_index,
}


@implementer(textindexer.IDynamicTextIndexExtender)
@adapter(IDexterityContainer)
class FolderishItemTextExtender:
    def __init__(self, context):
        self.context = context

    def __call__(self):
        layout = self.context.getLayout()
        if layout == "dynamic-view":
            return get_available_text_from_dynamic_pages(self.context)
        return ""


def get_enabled_fields(row_type):
    """return the fields that are enabled to be edited in the given
    row type
    """
    row_type_fields = api.portal.get_registry_record(
        "cs_dynamicpages.dynamic_pages_control_panel.row_type_fields"
    )

    for item in row_type_fields:
        if item["row_type"] == row_type:
            return item["each_row_type_fields"]

    return []


def get_available_text_from_dynamic_pages(context):
    """this should return the indexable texts
    for a given dynamic page

    it should extract the texts from the row container in the context
    """
    value = []
    brains = api.content.find(
        portal_type="DynamicPageRow",
        review_state="published",
        context=context,
    )
    for brain in brains:
        drr = brain.getObject()
        enabled_fields = get_enabled_fields(drr.row_type)
        for enabled_field in enabled_fields:
            extract_content_function = FIELDS_TO_INDEX.get(enabled_field)
            if extract_content_function is not None:
                value.append(extract_content_function(drr))

    return " ".join(value)



# from plone.app.z3cform.widgets.contentbrowser import ContentBrowserFieldWidget
from cs_dynamicpages import _
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import safe_hasattr
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


try:
    # This is for Plone 6.1
    from plone.app.z3cform.widgets.contentbrowser import (
        ContentBrowserFieldWidget as RelatedImageFieldWidget,
    )
except ImportError:
    # This is for previous versions of Plone
    from plone.app.z3cform.widgets.relateditems import (
        RelatedItemsFieldWidget as RelatedImageFieldWidget,
    )


class IRelatedImageMarker(Interface):
    pass


@provider(IFormFieldProvider)
class IRelatedImage(model.Schema):
    """ """

    related_image = RelationList(
        title=_("Related image"),
        description=_("Select the related image that will be shown in this row"),
        default=[],
        max_length=1,
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )

    form.widget(
        "related_image",
        RelatedImageFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "recentlyUsed": True,
            "selectableTypes": ["Image"],
            "upload": True,
        },
    )
    image_position = schema.Choice(
        title=_("Image position"),
        description=_("Select the position of this image"),
        vocabulary="cs_dynamicpages.ImagePosition",
        required=True,
        default="left",
    )


@implementer(IRelatedImage)
@adapter(IRelatedImageMarker)
class RelatedImage:
    def __init__(self, context):
        self.context = context

    @property
    def related_image(self):
        if safe_hasattr(self.context, "related_image"):
            return self.context.related_image
        return None

    @related_image.setter
    def related_image(self, value):
        self.context.related_image = value

    @property
    def image_position(self):
        if safe_hasattr(self.context, "image_position"):
            return self.context.image_position
        return None

    @image_position.setter
    def image_position(self, value):
        self.context.image_position = value



from AccessControl.SecurityManagement import newSecurityManager
from cs_dynamicpages.interfaces import IBrowserLayer
from Products.CMFPlone.factory import _DEFAULT_PROFILE
from Products.CMFPlone.factory import addPloneSite
from Products.GenericSetup.tool import SetupTool
from Testing.makerequest import makerequest
from zope.interface import directlyProvidedBy
from zope.interface import directlyProvides

import os
import transaction


truthy = frozenset(("t", "true", "y", "yes", "on", "1"))


def asbool(s):
    """Return the boolean value ``True`` if the case-lowered value of string
    input ``s`` is a :term:`truthy string`. If ``s`` is already one of the
    boolean values ``True`` or ``False``, return it."""
    if s is None:
        return False
    if isinstance(s, bool):
        return s
    s = str(s).strip()
    return s.lower() in truthy


DELETE_EXISTING = asbool(os.getenv("DELETE_EXISTING"))

app = makerequest(globals()["app"])

request = app.REQUEST

ifaces = [IBrowserLayer]
for iface in directlyProvidedBy(request):
    ifaces.append(iface)

directlyProvides(request, *ifaces)

admin = app.acl_users.getUserById("admin")
admin = admin.__of__(app.acl_users)
newSecurityManager(None, admin)

site_id = "Plone"
payload = {
    "title": "cs_dynamicpages",
    "profile_id": _DEFAULT_PROFILE,
    "distribution_name": "classic",
    "setup_content": False,
    "default_language": "en",
    "portal_timezone": "UTC",
}

if site_id in app.objectIds() and DELETE_EXISTING:
    app.manage_delObjects([site_id])
    transaction.commit()
    app._p_jar.sync()

if site_id not in app.objectIds():
    site = addPloneSite(app, site_id, **payload)
    transaction.commit()

    portal_setup: SetupTool = site.portal_setup
    portal_setup.runAllImportStepsFromProfile("profile-cs_dynamicpages:default")
    transaction.commit()



from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles:
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            "cs_dynamicpages:uninstall",
        ]

    def getNonInstallableProducts(self):
        """Hide the upgrades package from site-creation and quickinstaller."""
        return ["cs_dynamicpages.upgrades"]



# from cs_dynamicpages import _
from cs_dynamicpages.views.dynamic_page_row_view import DynamicPageRowView
from zope.interface import implementer
from zope.interface import Interface


class IFeaturedView(Interface):
    """Marker Interface for IFeaturedView"""


@implementer(IFeaturedView)
class FeaturedView(DynamicPageRowView):
    def related_image(self):
        related_image = self.context.related_image
        if related_image:
            return related_image[0].to_object
        return None



from Acquisition import aq_parent
from cs_dynamicpages import logger


def handler(obj, event):
    """When modifying a DynamicPageRow, we need to index the contents of the
    item where this row is shown.
    To do so we go up in the tree until we reach the content and force
    the reindex of it.
    """
    dynamic_page_folder = aq_parent(obj)
    if dynamic_page_folder.portal_type == "DynamicPageFolder":
        content = aq_parent(dynamic_page_folder)
        content.reindexObject()
        logger.info("Reindex item: %s", "/".join(content.getPhysicalPath()))



from cs_dynamicpages import logger
from cs_dynamicpages import PACKAGE_NAME

import pytest


class TestBase:
    @pytest.fixture(autouse=True)
    def installed(self, portal, installer):
        """
        Workaround to test isolation problems when using
        collective.z3cform.datagridfield

        See: https://community.plone.org/t/test-isolation-errors-with-collective-z3cform-datagridfield/7424
        """
        try:
            installer.install_product(PACKAGE_NAME)
        except Exception as e:
            logger.info(e)
            logger.info("Package already installed")



# from cs_dynamicpages import _
from Products.Five.browser import BrowserView
from zope.interface import implementer
from zope.interface import Interface


# from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class IDynamicPageRowView(Interface):
    """Marker Interface for IDynamicPageRowView"""


@implementer(IDynamicPageRowView)
class DynamicPageRowView(BrowserView):
    # If you want to define a template here, please remove the template from
    # the configure.zcml registration of this view.
    # template = ViewPageTemplateFile('dynamic_page_row_view.pt')

    def __call__(self):
        # Implement your own actions:
        return self.index()



from . import logger
from cs_dynamicpages.controlpanels.dynamic_pages_control_panel.controlpanel import (
    IDynamicPagesControlPanel,
)
from plone import api
from plone.app.upgrade.utils import alias_module
from zope.annotation.interfaces import IAnnotations


# from plone import api
import json


alias_module(
    "cs_dynamicpages.controlpanels.dynamica_pages_control_panel.controlpanel.IDynamicaPagesControlPanel",
    IDynamicPagesControlPanel,
)


def upgrade(setup_tool=None):
    """ """
    logger.info("Running upgrade (Python): Change Control Panel name in registry")


UPGRADEABLE_KEYS = ["row_type_fields", "row_widths"]


def pre_handler(setup_tool=None):
    """ """
    for key in UPGRADEABLE_KEYS:
        value = api.portal.get_registry_record(
            f"cs_dynamicpages.dynamica_pages_control_panel.{key}", default=[]
        )

        value_str = json.dumps(value)
        portal = api.portal.get()
        IAnnotations(portal)[
            f"cs_dynamicpages.dynamic_pages_control_panel.{key}.UPGRADE"
        ] = value_str

    logger.info("Save existing values for upgrade")


def post_handler(setup_tool=None):
    """ """
    portal = api.portal.get()
    for key in UPGRADEABLE_KEYS:
        annotated = IAnnotations(portal)
        value_str = annotated.get(
            f"cs_dynamicpages.dynamic_pages_control_panel.{key}.UPGRADE", "[]"
        )
        try:
            value = json.loads(value_str)
        except Exception:
            value = []
        if not isinstance(value, list):
            value = []
        api.portal.set_registry_record(
            f"cs_dynamicpages.dynamic_pages_control_panel.{key}", value
        )

        del annotated[f"cs_dynamicpages.dynamic_pages_control_panel.{key}.UPGRADE"]

    logger.info("Restored existing values after upgrade")



from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.registry import DictRow
from cs_dynamicpages import _
from cs_dynamicpages.interfaces import IBrowserLayer
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.autoform.directives import widget
from plone.restapi.controlpanels import RegistryConfigletPanel
from plone.z3cform import layout
from zope import schema
from zope.component import adapter
from zope.interface import Interface


class IRowTypeFieldsSchema(Interface):
    row_type = schema.Choice(
        title=_("Row type"),
        description=_(
            "Select the row type. This is the type of the row that will be added."
        ),
        required=True,
        vocabulary="cs_dynamicpages.RowType",
    )

    each_row_type_fields = schema.List(
        title=_("Row fields"),
        description=_(
            "Enter the fields that will be available when "
            "editing this row type. This is useful to hide unused fields."
        ),
        required=True,
        value_type=schema.TextLine(),
        default=[],
    )

    row_type_has_featured_add_button = schema.Bool(
        title=_("Has featured add button?"),
        description=_(
            "If selected a 'Add featured' button will be added in the edit "
            "interface. This is useful for rows that have content pieces "
            "inside them. For example in a slider row, there are slider items. "
            "This button will be used to add those items."
        ),
        required=False,
        default=False,
    )

    row_type_icon = schema.TextLine(
        title=_("Row type icon"),
        description=_("Icon for the row type"),
        required=True,
        default="bricks",
    )


class IRowWidthSchema(Interface):
    row_width_label = schema.TextLine(
        title=_("Row Width Label"),
        description=_("This is the label corresponding to this width"),
        required=True,
    )

    row_width_class = schema.TextLine(
        title=_("Row Width CSS class"),
        description=_("CSS class for the row width"),
        required=True,
    )


class ISpacerSchema(Interface):
    spacer_label = schema.TextLine(
        title=_("Spacer Label"),
        description=_("This is the label corresponding to this spacer"),
        required=True,
    )

    spacer_class = schema.TextLine(
        title=_("Spacer CSS class"),
        description=_("CSS class for the spacer"),
        required=True,
    )


class IDynamicPagesControlPanel(Interface):
    widget(row_type_fields=DataGridFieldFactory)
    row_type_fields = schema.List(
        title=_("Row type fields"),
        description=_(
            "Here we have all the available views for the rows and their settings"
        ),
        required=True,
        value_type=DictRow(title=_("Row type field"), schema=IRowTypeFieldsSchema),
        default=[
            {
                "row_type": "cs_dynamicpages-title-description-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IBasic.description",
                    "IRowWidth.width",
                    "IExtraClass.extra_class",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                ],
                "row_type_has_featured_add_button": False,
                "row_type_icon": "fonts",
            },
            {
                "row_type": "cs_dynamicpages-featured-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IBasic.description",
                    "IRowWidth.width",
                    "IExtraClass.extra_class",
                    "IRelatedImage.related_image",
                    "IFetchPriorityImage.fetchpriority_image",
                    "IRelatedImage.image_position",
                    "ILinkInfo.link_text",
                    "ILinkInfo.link_url",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                ],
                "row_type_has_featured_add_button": False,
                "row_type_icon": "card-image",
            },
            {
                "row_type": "cs_dynamicpages-featured-overlay-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IBasic.description",
                    "IRowWidth.width",
                    "IExtraClass.extra_class",
                    "IRelatedImage.related_image",
                    "IFetchPriorityImage.fetchpriority_image",
                    "ILinkInfo.link_text",
                    "ILinkInfo.link_url",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                ],
                "row_type_has_featured_add_button": False,
                "row_type_icon": "image-fill",
            },
            {
                "row_type": "cs_dynamicpages-horizontal-rule-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IRowWidth.width",
                    "IExtraClass.extra_class",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                ],
                "row_type_has_featured_add_button": False,
                "row_type_icon": "hr",
            },
            {
                "row_type": "cs_dynamicpages-spacer-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IExtraClass.extra_class",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                ],
                "row_type_has_featured_add_button": False,
                "row_type_icon": "arrows-vertical",
            },
            {
                "row_type": "cs_dynamicpages-slider-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IRowWidth.width",
                    "IExtraClass.extra_class",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                    "IFetchPriorityImage.fetchpriority_image",
                ],
                "row_type_has_featured_add_button": True,
                "row_type_icon": "images",
            },
            {
                "row_type": "cs_dynamicpages-features-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IRowWidth.width",
                    "IRowColumns.columns",
                    "IExtraClass.extra_class",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                    "IFetchPriorityImage.fetchpriority_image",
                ],
                "row_type_has_featured_add_button": True,
                "row_type_icon": "grid",
            },
            {
                "row_type": "cs_dynamicpages-accordion-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IRowWidth.width",
                    "IExtraClass.extra_class",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                ],
                "row_type_has_featured_add_button": True,
                "row_type_icon": "chevron-double-down",
            },
            {
                "row_type": "cs_dynamicpages-query-columns-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IRowWidth.width",
                    "IExtraClass.extra_class",
                    "ICollection.query",
                    "ICollection.sort_on",
                    "ICollection.sort_order",
                    "ICollection.betweeen",
                    "ICollection.limit",
                    "IRowColumns.columns",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                    "IFetchPriorityImage.fetchpriority_image",
                ],
                "row_type_has_featured_add_button": False,
                "row_type_icon": "funnel",
            },
            {
                "row_type": "cs_dynamicpages-text-view",
                "each_row_type_fields": [
                    "IBasic.title",
                    "IRowWidth.width",
                    "IExtraClass.extra_class",
                    "IRichTextBehavior-text",
                    "IRowVerticalSpacing.padding_top",
                    "IRowVerticalSpacing.padding_bottom",
                    "IRowVerticalSpacing.margin_top",
                    "IRowVerticalSpacing.margin_bottom",
                ],
                "row_type_has_featured_add_button": False,
                "row_type_icon": "body-text",
            },
        ],
    )

    widget(row_widths=DataGridFieldFactory)
    row_widths = schema.List(
        title=_("Row widths"),
        description=_("Here you can define the available widths for each row"),
        required=True,
        value_type=DictRow(
            title=_("Row width"),
            schema=Interface(
                IRowWidthSchema,
            ),
        ),
        default=[
            {
                "row_width_label": "Narrow",
                "row_width_class": "col-md-6 offset-md-3",
            },
            {
                "row_width_label": "Centered",
                "row_width_class": "col-md-8 offset-md-2",
            },
            {
                "row_width_label": "Full width",
                "row_width_class": "col-md-12",
            },
        ],
    )
    widget(spacer_padding_top=DataGridFieldFactory)
    spacer_padding_top = schema.List(
        title=_("Spacer padding top"),
        description=_("Here you can define the available paddings for each spacer"),
        required=True,
        value_type=DictRow(
            title=_("Spacer padding top"),
            schema=Interface(
                ISpacerSchema,
            ),
        ),
        default=[
            {
                "spacer_label": "0",
                "spacer_class": "pt-0",
            },
            {
                "spacer_label": "1",
                "spacer_class": "pt-1",
            },
            {
                "spacer_label": "2",
                "spacer_class": "pt-2",
            },
            {
                "spacer_label": "3",
                "spacer_class": "pt-3",
            },
            {
                "spacer_label": "4",
                "spacer_class": "pt-4",
            },
            {
                "spacer_label": "5",
                "spacer_class": "pt-5",
            },
        ],
    )

    widget(spacer_padding_bottom=DataGridFieldFactory)
    spacer_padding_bottom = schema.List(
        title=_("Spacer padding bottom"),
        description=_("Here you can define the available paddings for each spacer"),
        required=True,
        value_type=DictRow(
            title=_("Spacer padding bottom"),
            schema=Interface(
                ISpacerSchema,
            ),
        ),
        default=[
            {
                "spacer_label": "0",
                "spacer_class": "pb-0",
            },
            {
                "spacer_label": "1",
                "spacer_class": "pb-1",
            },
            {
                "spacer_label": "2",
                "spacer_class": "pb-2",
            },
            {
                "spacer_label": "3",
                "spacer_class": "pb-3",
            },
            {
                "spacer_label": "4",
                "spacer_class": "pb-4",
            },
            {
                "spacer_label": "5",
                "spacer_class": "pb-5",
            },
        ],
    )

    widget(spacer_margin_top=DataGridFieldFactory)
    spacer_margin_top = schema.List(
        title=_("Spacer margin top"),
        description=_("Here you can define the available margins for each spacer"),
        required=True,
        value_type=DictRow(
            title=_("Spacer margin top"),
            schema=Interface(
                ISpacerSchema,
            ),
        ),
        default=[
            {
                "spacer_label": "0",
                "spacer_class": "mt-0",
            },
            {
                "spacer_label": "1",
                "spacer_class": "mt-1",
            },
            {
                "spacer_label": "2",
                "spacer_class": "mt-2",
            },
            {
                "spacer_label": "3",
                "spacer_class": "mt-3",
            },
            {
                "spacer_label": "4",
                "spacer_class": "mt-4",
            },
            {
                "spacer_label": "5",
                "spacer_class": "mt-5",
            },
        ],
    )

    widget(spacer_margin_bottom=DataGridFieldFactory)
    spacer_margin_bottom = schema.List(
        title=_("Spacer margin bottom"),
        description=_("Here you can define the available margins for each spacer"),
        required=True,
        value_type=DictRow(
            title=_("Spacer margin bottom"),
            schema=Interface(
                ISpacerSchema,
            ),
        ),
        default=[
            {
                "spacer_label": "0",
                "spacer_class": "mb-0",
            },
            {
                "spacer_label": "1",
                "spacer_class": "mb-1",
            },
            {
                "spacer_label": "2",
                "spacer_class": "mb-2",
            },
            {
                "spacer_label": "3",
                "spacer_class": "mb-3",
            },
            {
                "spacer_label": "4",
                "spacer_class": "mb-4",
            },
            {
                "spacer_label": "5",
                "spacer_class": "mb-5",
            },
        ],
    )


class DynamicPagesControlPanel(RegistryEditForm):
    schema = IDynamicPagesControlPanel
    schema_prefix = "cs_dynamicpages.dynamic_pages_control_panel"
    label = _("Dynamic Pages Control Panel")


DynamicPagesControlPanelView = layout.wrap_form(
    DynamicPagesControlPanel, ControlPanelFormWrapper
)


@adapter(Interface, IBrowserLayer)
class DynamicPagesControlPanelConfigletPanel(RegistryConfigletPanel):
    """Control Panel endpoint"""

    schema = IDynamicPagesControlPanel
    configlet_id = "dynamic_pages_control_panel-controlpanel"
    configlet_category_id = "Products"
    title = _("Dynamic Pages Control Panel")
    group = ""
    schema_prefix = "cs_dynamicpages.dynamic_pages_control_panel"



# from plone import api
from plone import api
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class VocabItem:
    def __init__(self, token, value):
        self.token = token
        self.value = value


@implementer(IVocabularyFactory)
class RowPaddingTop:
    """ """

    def __call__(self, context):
        # Just an example list of content for our vocabulary,
        # this can be any static or dynamic data, a catalog result for example.
        values = api.portal.get_registry_record(
            "cs_dynamicpages.dynamic_pages_control_panel.spacer_padding_top",
            default=[],
        )

        terms = []
        for item in values:
            terms.append(
                SimpleTerm(
                    value=item["spacer_class"],
                    token=str(item["spacer_class"]),
                    title=item["spacer_label"],
                )
            )
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


RowPaddingTopFactory = RowPaddingTop()


@implementer(IVocabularyFactory)
class RowPaddingBottom:
    """ """

    def __call__(self, context):
        # Just an example list of content for our vocabulary,
        # this can be any static or dynamic data, a catalog result for example.
        values = api.portal.get_registry_record(
            "cs_dynamicpages.dynamic_pages_control_panel.spacer_padding_bottom",
            default=[],
        )

        terms = []
        for item in values:
            terms.append(
                SimpleTerm(
                    value=item["spacer_class"],
                    token=str(item["spacer_class"]),
                    title=item["spacer_label"],
                )
            )
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


RowPaddingBottomFactory = RowPaddingBottom()


@implementer(IVocabularyFactory)
class RowMarginTop:
    """ """

    def __call__(self, context):
        # Just an example list of content for our vocabulary,
        # this can be any static or dynamic data, a catalog result for example.
        values = api.portal.get_registry_record(
            "cs_dynamicpages.dynamic_pages_control_panel.spacer_margin_top",
            default=[],
        )

        terms = []
        for item in values:
            terms.append(
                SimpleTerm(
                    value=item["spacer_class"],
                    token=str(item["spacer_class"]),
                    title=item["spacer_label"],
                )
            )
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


RowMarginTopFactory = RowMarginTop()


@implementer(IVocabularyFactory)
class RowMarginBottom:
    """ """

    def __call__(self, context):
        # Just an example list of content for our vocabulary,
        # this can be any static or dynamic data, a catalog result for example.
        values = api.portal.get_registry_record(
            "cs_dynamicpages.dynamic_pages_control_panel.spacer_margin_bottom",
            default=[],
        )

        terms = []
        for item in values:
            terms.append(
                SimpleTerm(
                    value=item["spacer_class"],
                    token=str(item["spacer_class"]),
                    title=item["spacer_label"],
                )
            )
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


RowMarginBottomFactory = RowMarginBottom()



from . import logger
from cs_dynamicpages.utils import enable_behavior
from plone import api


def upgrade(setup_tool=None):
    """ """
    logger.info("Running upgrade (Python): Add new behavior")
    enable_behavior("cs_dynamicpages.fetchpriority_image")
    row_types_fields = api.portal.get_registry_record(
        "cs_dynamicpages.dynamic_pages_control_panel.row_type_fields", default=[]
    )
    new_row_types_fields = []

    for row_type_field in row_types_fields:
        if row_type_field["row_type"] in [
            "cs_dynamicpages-slider-view",
            "cs_dynamicpages-features-view",
            "cs_dynamicpages-query-columns-view",
            "cs_dynamicpages-featured-overlay-view",
            "cs_dynamicpages-featured-view",
        ]:
            each_row_type_fields = row_type_field.get("each_row_type_fields", [])
            each_row_type_fields.append("IFetchPriorityImage.fetchpriority_image")
            row_type_field["each_row_type_fields"] = each_row_type_fields
        new_row_types_fields.append(row_type_field)
    api.portal.set_registry_record(
        "cs_dynamicpages.dynamic_pages_control_panel.row_type_fields",
        new_row_types_fields,
    )



# from cs_dynamicpages.testing import CS_DYNAMICPAGES_FUNCTIONAL_TESTING
from cs_dynamicpages.testing import CS_DYNAMICPAGES_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class UpgradeStepIntegrationTest(unittest.TestCase):
    layer = CS_DYNAMICPAGES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_upgrade_step(self):
        # dummy, add tests here
        self.assertTrue(True)


# class UpgradeStepFunctionalTest(unittest.TestCase):
#
#     layer = CS_DYNAMICPAGES_FUNCTIONAL_TESTING
#
#     def setUp(self):
#         self.portal = self.layer['portal']
#         setRoles(self.portal, TEST_USER_ID, ['Manager'])



from cs_dynamicpages.content.dynamic_page_row import IDynamicPageRow
from zope.component import getSiteManager
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import providedBy
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class VocabItem:
    def __init__(self, token, value):
        self.token = token
        self.value = value


VIEW_PREFIX = "cs_dynamicpages-"


@implementer(IVocabularyFactory)
class RowType:
    """ """

    def __call__(self, context):
        items = []
        terms = []

        sm = getSiteManager()

        available_views = sm.adapters.lookupAll(
            required=(IDynamicPageRow, providedBy(getRequest())),
            provided=Interface,
        )

        available_view_names = [
            view[0] for view in available_views if view[0].startswith(VIEW_PREFIX)
        ]
        for view_name in available_view_names:
            items.append(VocabItem(view_name, view_name.replace(VIEW_PREFIX, "")))

        for item in items:
            terms.append(
                SimpleTerm(
                    value=item.token,
                    token=str(item.token),
                    title=item.value,
                )
            )
        return SimpleVocabulary(terms)


RowTypeFactory = RowType()



from cs_dynamicpages import logger
from plone import api
from plone.app.uuid.utils import uuidToObject
from urllib.parse import urlparse
from zope.component import getSiteManager
from zope.globalrequest import getRequest
from zope.interface import Interface
from zope.interface import providedBy


VIEW_PREFIX = "cs_dynamicpages-"

# links starting with these URL scheme should not be redirected to
NON_REDIRECTABLE_URL_SCHEMES = [
    "mailto:",
    "tel:",
    "callto:",  # nonstandard according to RFC 3966. used for skype.
    "webdav:",
    "caldav:",
]

# links starting with these URL scheme should not be resolved to paths
NON_RESOLVABLE_URL_SCHEMES = [*NON_REDIRECTABLE_URL_SCHEMES, "file:", "ftp:"]


def add_custom_view(
    view_name: str,
    shown_fields: list[str],
    has_button: bool = False,
    icon: str = "bricks",
):
    """utility function to add a given view to the list of available row types"""
    record_name = "cs_dynamicpages.dynamic_pages_control_panel.row_type_fields"
    values = api.portal.get_registry_record(record_name)
    new_item = {
        "row_type": view_name,
        "each_row_type_fields": shown_fields,
        "row_type_has_featured_add_button": has_button,
        "row_type_icon": icon,
    }
    values.append(new_item)
    api.portal.set_registry_record(record_name, values)
    logger.info("Added new row type: %s", view_name)

    return True


def enable_behavior(behavior_dotted_name=str):
    """
    utility function to enable the given behavior in the DynamicPageRow content type
    """
    # Get the portal_types tool, which manages all content type definitions (FTIs)
    portal_types = api.portal.get_tool("portal_types")

    # Get the Factory Type Information (FTI) for our specific content type
    fti = getattr(portal_types, "DynamicPageRow", None)

    if not fti:
        # Failsafe in case the content type doesn't exist
        print("Content type 'DynamicPageRow' not found.")
        return

    # Get the current list of behaviors
    behaviors = list(fti.behaviors)

    # --- The Core Logic ---
    # Check if the behavior is already enabled to avoid duplicates
    if behavior_dotted_name not in behaviors:
        print(f"Enabling behavior '{behavior_dotted_name}' on 'DynamicPageRow'.")
        # Add the new behavior to the list
        behaviors.append(behavior_dotted_name)
        # Assign the updated list back to the FTI's behaviors attribute
        fti.behaviors = tuple(behaviors)
    else:
        print(
            f"Behavior '{behavior_dotted_name}' is already enabled on 'DynamicPageRow'."
        )


def get_available_views_for_row():
    from cs_dynamicpages.content.dynamic_page_row import IDynamicPageRow

    items = []
    sm = getSiteManager()

    available_views = sm.adapters.lookupAll(
        required=(IDynamicPageRow, providedBy(getRequest())),
        provided=Interface,
    )

    values = api.portal.get_registry_record(
        "cs_dynamicpages.dynamic_pages_control_panel.row_type_fields", default=[]
    )

    for item in available_views:
        if item[0].startswith(VIEW_PREFIX):
            for value in values:
                item_dict = {
                    "row_type": item[0],
                    "each_row_type_fields": [],
                    "row_type_has_featured_add_button": False,
                    "row_type_icon": "bricks",
                }
                if item[0] == value["row_type"] and value not in items:
                    item_dict = value
                    items.append(item_dict)
    return items


def normalize_uid_from_path(url=None):
    """
    Args:
        url (string): a path or orl

    Returns:
        tuple: tuple of (uid, fragment) a fragment is an anchor id e.g. #head1
    """
    uid = None
    fragment = None

    if not url:
        return uid, fragment

    # resolve uid
    paths = url.split("/")
    paths_lower = [_item.lower() for _item in paths]

    if "resolveuid" in paths_lower:
        ri = paths_lower.index("resolveuid")
        if ri + 1 != len(paths):
            uid = paths[ri + 1]
            if uid == "":
                uid = None

    if not uid:
        return uid, fragment

    # resolve fragment
    parts = urlparse(uid)

    uid = parts.path

    fragment = f"#{parts.fragment}" if parts.fragment else None

    return uid, fragment


def _url_uses_scheme(schemes, url):
    return any(url.startswith(scheme) for scheme in schemes)


def absolute_target_url(url):
    """Compute the absolute target URL."""

    if _url_uses_scheme(NON_RESOLVABLE_URL_SCHEMES, url):
        # For non http/https url schemes, there is no path to resolve.
        return url

    else:
        if "resolveuid" in url:
            uid, fragment = normalize_uid_from_path(url)
            obj = uuidToObject(uid)
            if obj is None:
                # uid can't resolve, return the url
                return url

            url = obj.absolute_url()
            if fragment is not None:
                url = f"{url}{fragment}"
    return url



# from plone.app.textfield import RichText
# from plone.autoform import directives
from cs_dynamicpages.utils import absolute_target_url
from logging import getLogger
from plone import api
from plone.app.contenttypes.utils import replace_link_variables_by_paths
from plone.dexterity.content import Container

# from plone.namedfile import field as namedfile
from plone.supermodel import model

# from plone.supermodel.directives import fieldset
# from z3c.form.browser.radio import RadioFieldWidget
from zope import schema
from zope.interface import implementer


log = getLogger(__name__)

# from cs_dynamicpages import _


class IDynamicPageRow(model.Schema):
    """Marker interface and Dexterity Python Schema for DynamicPageRow"""

    row_type = schema.Choice(
        title="Row type",
        required=True,
        default="cs_dynamicpages-horizontal-rule-view",
        vocabulary="cs_dynamicpages.RowType",
    )


@implementer(IDynamicPageRow)
class DynamicPageRow(Container):
    """Content-type class for IPortadakoLerroa"""

    def row_template(self):
        return self.row_type.replace("cs_dynamicpages-", "")

    def review_state(self):
        return api.content.get_state(obj=self)

    def can_edit(self):
        return api.user.has_permission("Modify portal content", obj=self)

    def featured_list(self):
        return api.content.find(
            context=self,
            portal_type="DynamicPageRowFeatured",
            sort_on="getObjPositionInParent",
            depth=1,
        )

    def show_featured_add_button(self):
        row_type = self.row_type
        row_type_fields = api.portal.get_registry_record(
            "cs_dynamicpages.dynamic_pages_control_panel.row_type_fields"
        )
        for row_type_field in row_type_fields:
            if row_type_field["row_type"] == row_type:
                return row_type_field["row_type_has_featured_add_button"]
        return False

    def render(self, request):
        if self.row_type:
            try:
                view = api.content.get_view(
                    name=self.row_type,
                    context=self,
                    request=request,
                )
                return view()
            except Exception as e:
                log.error(e)
                return "ERROR RENDERING THE SELECTED ROW TYPE"
        return "THE SELECTED ROW TYPE IS NO LONGER AVAILABLE"

    def url(self):
        """Returns the url with link variables replaced."""
        url = replace_link_variables_by_paths(self, self.link_url.strip())
        return absolute_target_url(url)



from . import logger
from .base import reload_gs_profile


# from plone import api


def upgrade(setup_tool=None):
    """ """
    logger.info("Running upgrade (Python): Add a new default behavior to the row")
    reload_gs_profile(setup_tool)



from cs_dynamicpages import _
from cs_dynamicpages.testing import CS_DYNAMICPAGES_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.interfaces import IVocabularyTokenized

import unittest


class RowTypeIntegrationTest(unittest.TestCase):
    layer = CS_DYNAMICPAGES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_vocab_row_type(self):
        vocab_name = "cs_dynamicpages.RowType"
        factory = getUtility(IVocabularyFactory, vocab_name)
        self.assertTrue(IVocabularyFactory.providedBy(factory))

        vocabulary = factory(self.portal)
        self.assertTrue(IVocabularyTokenized.providedBy(vocabulary))
        self.assertEqual(
            vocabulary.getTerm("sony-a7r-iii").title,
            _("Sony Aplha 7R III"),
        )



from cs_dynamicpages import _
from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import safe_hasattr
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class IRowVerticalSpacingMarker(Interface):
    pass


@provider(IFormFieldProvider)
class IRowVerticalSpacing(model.Schema):
    padding_top = schema.Choice(
        title=_("Row padding top"),
        description=_("Select the padding top that this row will have"),
        vocabulary="cs_dynamicpages.RowPaddingTop",
        required=True,
        default="pt-0",
    )
    padding_bottom = schema.Choice(
        title=_("Row padding bottom"),
        description=_("Select the padding bottom that this row will have"),
        vocabulary="cs_dynamicpages.RowPaddingBottom",
        required=True,
        default="pb-0",
    )
    margin_top = schema.Choice(
        title=_("Row margin top"),
        description=_("Select the margin top that this row will have"),
        vocabulary="cs_dynamicpages.RowMarginTop",
        required=True,
        default="mt-0",
    )
    margin_bottom = schema.Choice(
        title=_("Row margin bottom"),
        description=_("Select the margin bottom that this row will have"),
        vocabulary="cs_dynamicpages.RowMarginBottom",
        required=True,
        default="mb-0",
    )


@implementer(IRowVerticalSpacing)
@adapter(IRowVerticalSpacingMarker)
class RowVerticalSpacing:
    def __init__(self, context):
        self.context = context

    @property
    def padding_top(self):
        if safe_hasattr(self.context, "padding_top"):
            return self.context.padding_top
        return None

    @padding_top.setter
    def padding_top(self, value):
        self.context.padding_top = value

    @property
    def padding_bottom(self):
        if safe_hasattr(self.context, "padding_bottom"):
            return self.context.padding_bottom
        return None

    @padding_bottom.setter
    def padding_bottom(self, value):
        self.context.padding_bottom = value

    @property
    def margin_top(self):
        if safe_hasattr(self.context, "margin_top"):
            return self.context.margin_top
        return None

    @margin_top.setter
    def margin_top(self, value):
        self.context.margin_top = value

    @property
    def margin_bottom(self):
        if safe_hasattr(self.context, "margin_bottom"):
            return self.context.margin_bottom
        return None

    @margin_bottom.setter
    def margin_bottom(self, value):
        self.context.margin_bottom = value



from cs_dynamicpages.testing import CS_DYNAMICPAGES_FUNCTIONAL_TESTING
from cs_dynamicpages.testing import CS_DYNAMICPAGES_INTEGRATION_TESTING
from cs_dynamicpages.views.featured_view import IFeaturedView
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getMultiAdapter
from zope.interface.interfaces import ComponentLookupError

import unittest


class ViewsIntegrationTest(unittest.TestCase):
    layer = CS_DYNAMICPAGES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        api.content.create(self.portal, "Folder", "other-folder")
        api.content.create(self.portal, "Document", "front-page")

    def test_featured_view_is_registered(self):
        view = getMultiAdapter(
            (self.portal["other-folder"], self.portal.REQUEST), name="featured-view"
        )
        self.assertTrue(IFeaturedView.providedBy(view))

    def test_featured_view_not_matching_interface(self):
        view_found = True
        try:
            view = getMultiAdapter(
                (self.portal["front-page"], self.portal.REQUEST), name="featured-view"
            )
        except ComponentLookupError:
            view_found = False
        else:
            view_found = IFeaturedView.providedBy(view)
        self.assertFalse(view_found)


class ViewsFunctionalTest(unittest.TestCase):
    layer = CS_DYNAMICPAGES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])



from ..base import TestBase
from plone import api

import pytest
import transaction


class TestContent(TestBase):
    @pytest.fixture(autouse=True)
    def create_content(self, portal):
        with api.env.adopt_roles(["Manager"]):
            self.folder = api.content.create(
                container=portal,
                type="Folder",
                id="folder",
            )

            assert self.folder is not None
            assert self.folder.id == "folder"

            self.folder.setLayout("dynamic-view")

            self.dpf = api.content.create(
                container=self.folder, type="DynamicPageFolder", id="dpf", title="DPF"
            )

            assert self.dpf is not None
            assert self.dpf.id == "dpf"

            self.row1 = api.content.create(
                container=self.dpf, type="DynamicPageRow", id="row-1", title="Row 1"
            )
            assert self.row1 is not None
            assert self.row1.id == "row-1"

            self.row2 = api.content.create(
                container=self.dpf, type="DynamicPageRow", id="row-2", title="Row 2"
            )
            assert self.row2 is not None
            assert self.row2.id == "row-2"

            transaction.commit()

    def test_view(self, browser):
        """check that the folder is rendered correctly with the basic instructions"""

        browser.open(self.folder.absolute_url())
        # We have 2 rows, so there must be an option to delete a row
        # assert "Delete row" in browser.contents

        # assert "Row 1" in browser.contents
        # assert "Row 2" in browser.contents

        # There must be an option to add a new row
        assert "Add row" in browser.contents

    # def test_add_row(self, browser):
    #     """click add row"""
    #     browser.open(self.folder.absolute_url())
    #     link = browser.getLink(title="Add row")
    #     link.click()
    #     assert "++add++DynamicPageRow" in browser.url

    #     control = browser.getControl(name="form.widgets.IBasic.title")
    #     control.value = "Row 3"

    #     save = browser.getControl(name="form.buttons.save")
    #     save.click()

    #     browser.open(self.folder.absolute_url())

    #     assert len(self.folder.dpf.keys()) == 3
